package billiardlibrary;

import java.util.Vector;
import java.util.Enumeration;

/**
 * @author End User
 */
public class Billiard2D {
	private Vector balls = new Vector();
	public double xfrom, xto, yfrom, yto;

	Billiard2D(double xfrom, double yfrom, double xto, double yto) {
		this.xfrom = xfrom;
		this.yfrom = yfrom;
		this.xto = xto;
		this.yto = yto;
	}

	int getBalls() {
		return balls.size();
	}

	void add(Ball b) {
		balls.addElement(b);
	}

	Ball get(int i) {
		return (Ball) balls.elementAt(i);
	}

	Ball get(String name) {
		Enumeration e = balls.elements();
		while (e.hasMoreElements()) {
			Ball b = (Ball) e.nextElement();
			if (name != null && name.equals(b.name)) {
				return b;
			}
		}
		return null;
	}

	void step() {
		for (int i = 0; i < getBalls(); i++) {
			get(i).step();
		}
	}

	private double new_vx(double dx, double dy, double vx1, double vy1,
			double vx2, double vy2) {
		if (dx+dy==0.0){ return Math.random()-.5; }
		return vx2 - dy * (dx * (vy1 - vy2) + dy * (vx2 - vx1))
				/ (dx * dx + dy * dy);
	}

	private double new_vy(double dx, double dy, double vx1, double vy1,
			double vx2, double vy2) {
		if (dx+dy==0.0){ return Math.random()-.5; }
		return vy1 - dy * (dx * (vx1 - vx2) + dy * (vy1 - vy2))
				/ (dx * dx + dy * dy);
	}

	private void updateBallsSpeedForCollision(Ball a, Ball b) {
		// if (a.collision > 0 || b.collision > 0) return;

		double dx = b.x - a.x;
		double dy = b.y - a.y;
		double v1x = new_vx(dx, dy, a.vx, a.vy, b.vx, b.vy);
		double v1y = new_vy(dx, dy, a.vx, a.vy, b.vx, b.vy);
		double v2x = new_vx(dx, dy, b.vx, b.vy, a.vx, a.vy);
		double v2y = new_vy(dx, dy, b.vx, b.vy, a.vx, a.vy);

		a.vx = v1x;
		a.vy = v1y;
		b.vx = v2x;
		b.vy = v2y;

		double e = 0.0001;
		double d1 = Math.sqrt(Math.pow(a.x - b.x, 2) + Math.pow(a.y - b.y, 2));
		double d2 = Math.sqrt(Math.pow(a.x + e * a.vx - (b.x + e * b.vx), 2)
				+ Math.pow(a.y + e * a.vy - (b.y + e * b.vy), 2));
		if (d2 < d1) {
			a.vx *= -1;
			a.vy *= -1;
			b.vx *= -1;
			b.vy *= -1;
		}

		// a.collision = 5;
		// b.collision = 5;
	}

	boolean updateBallsSpeedForCollision() {
		boolean coll = false;
		for (int i = 0; i < getBalls() - 1; i++) {
			double x1 = get(i).x;
			double y1 = get(i).y;
			double r1 = get(i).radius;
			for (int j = i + 1; j < getBalls(); j++) {
				if (Math.pow(x1 - get(j).x, 2) + Math.pow(y1 - get(j).y, 2) <= Math
						.pow(r1 + get(j).radius, 2)) {
					coll = true;
					updateBallsSpeedForCollision(get(i), get(j));
				}
			}
		}
		return coll;
	}

	private boolean updateBallSpeedForBorder(Ball b) {
		boolean coll = false;
		if (b.x + b.radius >= xto) {
			b.vx = -Math.abs(b.vx);
			coll = true;
		}
		if (b.x - b.radius <= xfrom) {
			b.vx = Math.abs(b.vx);
			coll = true;
		}
		if (b.y + b.radius >= yto) {
			b.vy = -Math.abs(b.vy);
			coll = true;
		}
		if (b.y - b.radius <= yfrom) {
			b.vy = Math.abs(b.vy);
			coll = true;
		}
		return coll;
	}

	boolean updateBallSpeedForBorder() {
		boolean coll = false;
		for (int i = 0; i < getBalls(); i++) {
			coll |= updateBallSpeedForBorder(get(i));
		}
		return coll;
	}

	double energy() {
		double sum = 0;
		for (int i = 0; i < getBalls(); i++) {
			sum += get(i).vx * get(i).vx + get(i).vy * get(i).vy;
		}
		return sum;
	}

	void applyFriction(double friction1, double eps, double friction2,
			double eps_stop) {
		double eps2 = eps * eps;
		double eps_stop2 = eps_stop * eps_stop;
		for (int i = 0; i < getBalls(); i++) {
			Ball b = get(i);
			double r = b.vx * b.vx + b.vy * b.vy;
			if (r <= eps_stop2) {
				b.vx = b.vy = 0.0;
			} else {
				double f = (r <= eps2) ? friction2 : friction1;
				b.vx *= (1.0 - f);
				b.vy *= (1.0 - f);
			}
		}
	}

	boolean isAllQuiet(double eps) {
		return energy() < eps * eps;
	}

}
